@@ -28,7 +28,7 @@ static time_t last_update;
 static struct iw_nl80211_linkstat *ls_tmp = NULL,
 				  *ls_cur = NULL,
 				  *ls_new = NULL;
-static pthread_mutex_t linkstat_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t linkstat_mutex;
 
 /** Sampling pthread - shared by info and histogram screen. */
 static void *sampling_loop(void *arg)
@@ -105,6 +105,10 @@ static void display_levels(void)
 	if (sig_level == 0)
 		sig_level = ls_cur->bss_signal;
 
+	/* If the signal level is positive, assume it is an absolute value (#100). */
+	if (sig_level > 0)
+		sig_level *= -1;
+
 	for (line = 1; line <= WH_LEVEL; line++)
 		mvwclrtoborder(w_levels, line, 1);
 
@@ -227,7 +231,7 @@ static void display_packet_counts(void)
 
 	if (ls_cur->rx_drop_misc) {
 		waddstr(w_stats, ", drop: ");
-		sprintf(tmp, "%'llu (%.1f%%)", (unsigned long long)ls_cur->rx_drop_misc,
+		sprintf(tmp, "%llu (%.2f%%)", (unsigned long long)ls_cur->rx_drop_misc,
 				(1e2 * ls_cur->rx_drop_misc)/ls_cur->rx_packets);
 		waddstr_b(w_stats, tmp);
 	}
@@ -254,7 +258,7 @@ static void display_packet_counts(void)
 
 	if (ls_cur->tx_retries) {
 		waddstr(w_stats, ", retries: ");
-		sprintf(tmp, "%s (%.1f%%)", int_counts(ls_cur->tx_retries),
+		sprintf(tmp, "%s (%.2f%%)", int_counts(ls_cur->tx_retries),
 			(1e2 * ls_cur->tx_retries)/ls_cur->tx_packets);
 		waddstr_b(w_stats, tmp);
 	}
@@ -277,11 +281,11 @@ static void display_interface(WINDOW *w_if, struct iw_nl80211_ifstat *ifs, bool
 
 	wmove(w_if, 1, 1);
 	waddstr_b(w_if, conf_ifname());
-	waddstr(w_if, "  ");
+	waddstr(w_if, " - ");
 
 	if (if_is_up) {
 		/* Wireless device index */
-		waddstr(w_if, "- wdev ");
+		waddstr(w_if, "wdev ");
 		sprintf(tmp, "%d", ifs->wdev);
 		waddstr_b(w_if, tmp);
 
@@ -355,7 +359,10 @@ static void display_info(WINDOW *w_info, struct iw_nl80211_ifstat *ifs)
 			waddstr_b(w_info, pretty_time(ls_cur->connected_time));
 
 			waddstr(w_info, ", inactive: ");
-			sprintf(tmp, "%.1fs", (float)ls_cur->inactive_time/1e3);
+			if  (ls_cur->inactive_time > 100)
+				sprintf(tmp, "%.1fs", (float)ls_cur->inactive_time/1e3);
+			else
+				sprintf(tmp, "%dms", ls_cur->inactive_time);
 			waddstr_b(w_info, tmp);
 		}
 	}
@@ -391,10 +398,10 @@ static void display_info(WINDOW *w_info, struct iw_nl80211_ifstat *ifs)
 		sprintf(tmp, "%d", ieee80211_frequency_to_channel(ifs->freq));
 		waddstr_b(w_info, tmp);
 
-		if (ifs->chan_width >= 0) {
+		if (ifs->chan_width != (uint32_t)-1) {
 			sprintf(tmp, " (width: %s)", channel_width_name(ifs->chan_width));
 			waddstr(w_info, tmp);
-		} else if (ifs->chan_type >= 0) {
+		} else if (ifs->chan_type != (uint32_t)-1) {
 			sprintf(tmp, " (%s)", channel_type_name(ifs->chan_type));
 			waddstr(w_info, tmp);
 		}
@@ -416,7 +423,7 @@ static void display_info(WINDOW *w_info, struct iw_nl80211_ifstat *ifs)
 	wmove(w_info, 3, 1);
 	if (ls_cur->beacons) {
 		waddstr(w_info, "beacons: ");
-		sprintf(tmp, "%'llu", (unsigned long long)ls_cur->beacons);
+		sprintf(tmp, "%llu", (unsigned long long)ls_cur->beacons);
 		waddstr_b(w_info, tmp);
 
 		if (ls_cur->beacon_loss) {
@@ -551,100 +558,219 @@ static void display_info(WINDOW *w_info, struct iw_nl80211_ifstat *ifs)
 }
 
 /** Network information pertaining to interface with interface index @ifindex. */
-static void display_netinfo(WINDOW *w_net, struct if_info *info, uint32_t ifindex)
+static void display_netinfo(WINDOW *w_net, struct if_info *info)
 {
+	struct if_info *active = info->master ? info->master : info;
 	char tmp[0x40];
 
 	wmove(w_net, 1, 1);
 	wclrtoborder(w_net);
-	if (getmaxy(w_net) == WH_NET_MAX) {
-		waddstr(w_net, conf_ifname());
-		sprintf(tmp, " (#%u, ", ifindex);
+
+	if (!ifinfo_is_up(info)) {
+		waddstr(w_net, info->ifname);
+
+		sprintf(tmp, " #%u ", info->ifindex);
 		waddstr(w_net, tmp);
 
-		if (info->flags & IFF_UP) {
-			waddstr(w_net, "UP");
+		waddstr_b(w_net, "(");
 
-			if (info->flags & IFF_RUNNING)		/* Interface RFC2863 OPER_UP	*/
-				waddstr(w_net, " RUNNING");
+		wadd_attr_str(w_net, COLOR_PAIR(CP_RED) | A_REVERSE,
+			!(info->flags & IFF_UP) ? "DOWN" : "no carrier");
+
+	} else if (info->master) {
+		waddstr_b(w_net, info->ifname);
+
+		sprintf(tmp, " #%u, %sslave of ", info->ifindex,
+			is_primary_slave(info->master->ifname, info->ifname)
+			? "primary " : "");
+		waddstr(w_net, tmp);
+
+		waddstr_b(w_net, info->master->ifname);
+		sprintf(tmp, " #%u ", info->master->ifindex);
+		waddstr(w_net, tmp);
+
+		waddstr_b(w_net, "(");
+
+		if (!(active->flags & IFF_UP)) {
+			waddstr(w_net, "DOWN");
+		} else {
+			const char *mode = NULL;
+
+			if (strcmp(info->master->type, "bond") == 0)
+				mode = get_bonding_mode(info->master->ifname);
+
+			if (mode) {
+				sprintf(tmp, "%s mode", mode);
+				waddstr(w_net, tmp);
+			} else if (*info->master->type) {
+				sprintf(tmp, "type %s", info->master->type);
+				waddstr(w_net, tmp);
+			} else {
+				waddstr(w_net, "UP");
+				if (active->flags & IFF_RUNNING)
+					waddstr(w_net, " RUNNING");
+				if (active->flags & IFF_MASTER)
+					waddstr(w_net, " MASTER");
+			}
+		}
+	} else {
+		waddstr(w_net, info->ifname);
+		sprintf(tmp, " #%u ", info->ifindex);
+		waddstr(w_net, tmp);
+		waddstr_b(w_net, "(");
+
+		waddstr(w_net, "UP");
+
+		if (info->flags & IFF_RUNNING)		/* Interface RFC2863 OPER_UP	*/
+			waddstr(w_net, " RUNNING");
 #ifdef IFF_LOWER_UP	/* Linux 2.6.17 */
-			if (info->flags & IFF_LOWER_UP)		/* Driver signals L1 up		*/
-				waddstr(w_net, " LOWER_UP");
+		if (info->flags & IFF_LOWER_UP)		/* Driver signals L1 up		*/
+			waddstr(w_net, " LOWER_UP");
 #endif
 #ifdef IFF_DORMANT	/* Linux 2.6.17 */
-			if (info->flags & IFF_DORMANT)		/* Driver signals dormant	*/
-				waddstr(w_net, " DORMANT");
+		if (info->flags & IFF_DORMANT)		/* Driver signals dormant	*/
+			waddstr(w_net, " DORMANT");
 #endif
-			if (info->flags & IFF_MASTER)		/* Master of a load balancer 	*/
-				waddstr(w_net, " MASTER");
-			if (info->flags & IFF_SLAVE)		/* Slave of a load balancer 	*/
-				waddstr(w_net, " SLAVE");
-			if (info->flags & IFF_POINTOPOINT)	/* Is a point-to-point link	*/
-				waddstr(w_net, " POINTOPOINT");
-			if (info->flags & IFF_DYNAMIC)		/* Address is volatile		*/
-				waddstr(w_net, " DYNAMIC");
-			if (info->flags & IFF_BROADCAST)		/* Valid broadcast address set	*/
-				waddstr(w_net, " BROADCAST");
-			if (info->flags & IFF_MULTICAST)		/* Supports multicast		*/
-				waddstr(w_net, " MULTICAST");
-			if (info->flags & IFF_ALLMULTI)		/* Receive all mcast  packets	*/
-				waddstr(w_net, " ALLMULTI");
-			if (info->flags & IFF_NOARP)		/* No ARP protocol		*/
-				waddstr(w_net, " NOARP");
-			if (info->flags & IFF_NOTRAILERS)	/* Avoid use of trailers	*/
-				waddstr(w_net, " NOTRAILERS");
-			if (info->flags & IFF_PROMISC)		/* Is in promiscuous mode	*/
-				waddstr(w_net, " PROMISC");
-			if (info->flags & IFF_DEBUG)		/* Internal debugging flag	*/
-				waddstr(w_net, " DEBUG");
-		} else {
-			wadd_attr_str(w_net, COLOR_PAIR(CP_RED) | A_REVERSE, "DOWN");
-		}
-		waddstr_b(w_net, ")");
-
-		wmove(w_net, 2, 1);
-		wclrtoborder(w_net);
+		if (info->flags & IFF_MASTER)		/* Master of a load balancer 	*/
+			waddstr(w_net, " MASTER");
+		if (info->flags & IFF_SLAVE)		/* Slave of a load balancer 	*/
+			waddstr(w_net, " SLAVE");
+		if (info->flags & IFF_POINTOPOINT)	/* Is a point-to-point link	*/
+			waddstr(w_net, " POINTOPOINT");
+		if (info->flags & IFF_DYNAMIC)		/* Address is volatile		*/
+			waddstr(w_net, " DYNAMIC");
+		if (info->flags & IFF_BROADCAST)	/* Valid broadcast address set	*/
+			waddstr(w_net, " BROADCAST");
+		if (info->flags & IFF_MULTICAST)	/* Supports multicast		*/
+			waddstr(w_net, " MULTICAST");
+		if (info->flags & IFF_ALLMULTI)		/* Receive all mcast  packets	*/
+			waddstr(w_net, " ALLMULTI");
+		if (info->flags & IFF_NOARP)		/* No ARP protocol		*/
+			waddstr(w_net, " NOARP");
+		if (info->flags & IFF_NOTRAILERS)	/* Avoid use of trailers	*/
+			waddstr(w_net, " NOTRAILERS");
+		if (info->flags & IFF_PROMISC)		/* Is in promiscuous mode	*/
+			waddstr(w_net, " PROMISC");
+		if (info->flags & IFF_DEBUG)		/* Internal debugging flag	*/
+			waddstr(w_net, " DEBUG");
 	}
-	waddstr(w_net, "mac: ");
-	waddstr_b(w_net, ether_lookup(&info->hwaddr));
+	waddstr_b(w_net, ")");
 
-	if (getmaxy(w_net) == WH_NET_MAX) {
-		if (info->flags & IFF_UP) {
-			waddstr(w_net, ", qlen: ");
-			sprintf(tmp, "%u", info->txqlen);
+	wmove(w_net, 2, 1);
+	wclrtoborder(w_net);
+
+	waddstr(w_net, "mode: ");
+	if (ifinfo_is_up(info)) {
+		waddstr_b(w_net, info->mode);
+
+		// Queueing discipline
+		if (*info->qdisc) {
+			waddstr(w_net, ", qdisc: ");
+			waddstr_b(w_net, info->qdisc);
+			if (info->master && *info->master->qdisc &&
+			    strcmp(info->master->qdisc, info->qdisc)) {
+				waddstr(w_net, "/");
+				waddstr_b(w_net, info->master->qdisc);
+			}
+		}
+
+		// Number of TX queues
+		if (info->numtxq > 1 || (info->master && info->master->numtxq > 1)) {
+			waddstr(w_net, ", txq: ");
+			sprintf(tmp, "%u", info->numtxq);
 			waddstr_b(w_net, tmp);
-
+			if (info->master && info->master->numtxq != info->numtxq) {
+				waddstr(w_net, "/");
+				sprintf(tmp, "%u", info->master->numtxq);
+				waddstr_b(w_net, tmp);
+			}
 		}
-		wmove(w_net, 3, 1);
-		wclrtoborder(w_net);
-	} else {
-		waddstr(w_net, ", ");
-	}
-	waddstr(w_net, "ip: ");
 
-	if (!info->addr.s_addr) {
-		waddstr(w_net, "n/a");
-	} else {
-		sprintf(tmp, "%s/%u", inet_ntoa(info->addr),
-				      prefix_len(&info->netmask));
+		// Queue lengths
+		waddstr(w_net, ", qlen: ");
+		sprintf(tmp, "%u", info->txqlen);
 		waddstr_b(w_net, tmp);
+		if (info->master && info->master->txqlen != info->txqlen) {
+			waddstr(w_net, "/");
+			sprintf(tmp, "%u", info->master->txqlen);
+			waddstr_b(w_net, tmp);
+		}
+	}
 
-		/* only show bcast address if not set to the obvious default */
-		if (info->bcast.s_addr !=
-		    (info->addr.s_addr | ~info->netmask.s_addr)) {
-			waddstr(w_net, ",  bcast: ");
-			waddstr_b(w_net, inet_ntoa(info->bcast));
+	wmove(w_net, 3, 1);
+	wclrtoborder(w_net);
+
+	/* Layer 2 information (display hardware address of active interface). */
+	waddstr(w_net, "mac: ");
+	if (ether_addr_is_zero(&active->hwaddr)) {
+		waddstr_b(w_net, "n/a");
+	} else if (!ifinfo_is_up(info)) {
+		wadd_attr_str(w_net, COLOR_PAIR(CP_RED) | A_REVERSE,
+			ether_lookup(&active->hwaddr));
+	} else {
+		waddstr_b(w_net, ether_lookup(&active->hwaddr));
+	}
+
+
+	if (ifinfo_is_up(info)) {
+		/* 802.11 MTU may be greater than Ethernet MTU (1500) */
+		if (info->mtu && info->mtu != ETH_DATA_LEN) {
+			waddstr(w_net, ", mtu: ");
+			sprintf(tmp, "%u", info->mtu);
+			waddstr_b(w_net, tmp);
+			if (info->master && info->master->mtu != info->mtu) {
+				waddstr(w_net, "/");
+				sprintf(tmp, "%u", info->master->mtu);
+				waddstr_b(w_net, tmp);
+			}
+		}
+	}
+
+	wclrtoborder(w_net);
+	wmove(w_net, 4, 1);
+
+	/* Layer 3 information of active interface */
+	waddstr(w_net, "ip4: ");
+	if (!*active->v4.addr) {
+		waddstr_b(w_net, "n/a");
+	} else {
+		waddstr_b(w_net, active->v4.addr);
+		if (active->v4.count > 1) {
+			sprintf(tmp, " (+%d)", active->v4.count-1);
+			waddstr(w_net, tmp);
+		}
+		if (active->v4.valid_lft && active->v4.valid_lft >= active->v4.preferred_lft) {
+			waddstr(w_net, ", valid: ");
+			waddstr_b(w_net, lft2str(active->v4.valid_lft));
+		} else if (active->v4.preferred_lft) {
+			waddstr(w_net, ", preferred: ");
+			waddstr_b(w_net, lft2str(active->v4.preferred_lft));
 		}
 	}
 	wclrtoborder(w_net);
 
-	/* 802.11 MTU may be greater than Ethernet MTU (1500) */
-	if (info->mtu && info->mtu != ETH_DATA_LEN) {
-		waddstr(w_net, ",  mtu: ");
-		sprintf(tmp, "%u", info->mtu);
-		waddstr_b(w_net, tmp);
+	wmove(w_net, 5, 1);
+	waddstr(w_net, "ip6: ");
+	if (!*active->v6.addr) {
+		waddstr_b(w_net, "n/a");
+	} else {
+		waddstr_b(w_net, active->v6.addr);
+		if (active->v6.count > 1) {
+			sprintf(tmp, " (+%d)", active->v6.count-1);
+			waddstr(w_net, tmp);
+		}
+		if (active->v6.valid_lft && active->v6.valid_lft >= active->v6.preferred_lft) {
+			waddstr(w_net, ", valid: ");
+			waddstr_b(w_net, lft2str(active->v6.valid_lft));
+		} else if (active->v6.preferred_lft) {
+			waddstr(w_net, ", preferred: ");
+			waddstr_b(w_net, lft2str(active->v6.preferred_lft));
+		}
 	}
 
+	free(info->master);
+
+	wclrtoborder(w_net);
 	wrefresh(w_net);
 }
 
@@ -658,7 +784,7 @@ static void display_static_parts(WINDOW *w_if, WINDOW *w_info, WINDOW *w_net)
 
 	display_interface(w_if, &ifs, net_info.flags & IFF_UP);
 
-	if (net_info.flags & IFF_UP) {
+	if (ifinfo_is_up(&net_info)) {
 		display_info(w_info, &ifs);
 	} else {
 		for (int i = 1; i <= WH_INFO; i++)
@@ -666,13 +792,14 @@ static void display_static_parts(WINDOW *w_if, WINDOW *w_info, WINDOW *w_net)
 	}
 	wrefresh(w_info);
 
-	display_netinfo(w_net, &net_info, ifs.ifindex);
+	display_netinfo(w_net, &net_info);
 }
 
 void scr_info_init(void)
 {
 	int line = 0;
 	bool ready = false;
+	static bool initialized = false;
 
 	w_if	 = newwin_title(line, WH_IFACE, "Interface", true);
 	line += WH_IFACE;
@@ -682,10 +809,16 @@ void scr_info_init(void)
 	line += WH_STATS;
 	w_info	 = newwin_title(line, WH_INFO, "Info", true);
 	line += WH_INFO;
-	if (LINES >= WH_INFO_SCR_MIN + (WH_NET_MAX - WH_NET_MIN))
-		w_net = newwin_title(line, WH_NET_MAX, "Network", false);
-	else
-		w_net = newwin_title(line, WH_NET_MIN, "Network", false);
+	w_net = newwin_title(line, WH_NET, "Network", false);
+
+	if (!initialized) {
+		pthread_mutexattr_t attr;
+
+		pthread_mutexattr_init(&attr);
+		pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
+		pthread_mutex_init(&linkstat_mutex, &attr);
+		initialized = true;
+	}
 
 	sampling_init(false);
 
@@ -720,6 +853,8 @@ int scr_info_loop(WINDOW *w_menu)
 
 void scr_info_fini(void)
 {
+	/* Unlock mutex in case it was taken when scr_info_loop got interrupted by a SIGWINCH. */
+	pthread_mutex_unlock(&linkstat_mutex);
 	sampling_stop();
 	last_update = 0;
 
