@@ -9,10 +9,29 @@
 
 #include "iw_nl80211.h"
 
+/** Append msg_attribute{type, len, data} to @cmd. */
+static void add_msg_arg(struct cmd *cmd, int type, size_t len, const void * const data)
+{
+	cmd->msg_args = realloc(cmd->msg_args, sizeof(*cmd->msg_args) * (cmd->msg_args_len + 1));
+	if (!cmd->msg_args)
+		err_sys("failed to allocate space for message attributes");
+
+	cmd->msg_args[cmd->msg_args_len].type = type;
+	cmd->msg_args[cmd->msg_args_len].len  = len;
+	cmd->msg_args[cmd->msg_args_len].data = data;
+	cmd->msg_args_len += 1;
+}
+
+static void free_msg_args(struct cmd *cmd)
+{
+	free(cmd->msg_args);
+	cmd->msg_args = NULL;
+	cmd->msg_args_len = 0;
+}
+
 /**
- * handle_cmd: process @cmd
+ * handle_cmd: process @cmd (generic variant)
  * Returns 0 if ok, -errno < 0 on failure
- * stolen/modified from iw:iw.c
  */
 int handle_cmd(struct cmd *cmd)
 {
@@ -20,13 +39,11 @@ int handle_cmd(struct cmd *cmd)
 	struct nl_msg *msg;
 	static int nl80211_id = -1;
 	int ret;
-	uint32_t ifindex, idx;
 
 	/*
 	 * Initialization of static components:
 	 * - per-cmd socket
 	 * - global nl80211 ID
-	 * - per-cmd interface index (in case conf_ifname() changes)
 	 */
 	if (!cmd->sk) {
 		cmd->sk = nl_socket_alloc();
@@ -44,10 +61,6 @@ int handle_cmd(struct cmd *cmd)
 			err_sys("nl80211 not found");
 	}
 
-	ifindex = if_nametoindex(conf_ifname());
-	if (ifindex == 0 && errno)
-		err_sys("failed to look up interface %s", conf_ifname());
-
 	/*
 	 * Message Preparation
 	 */
@@ -61,15 +74,13 @@ int handle_cmd(struct cmd *cmd)
 
 	genlmsg_put(msg, 0, 0, nl80211_id, 0, cmd->flags, cmd->cmd, 0);
 
-	/* netdev identifier: interface index */
-	NLA_PUT(msg, NL80211_ATTR_IFINDEX, sizeof(ifindex), &ifindex);
-
-	/* Additional attributes */
+	/* Message attributes */
 	if (cmd->msg_args) {
-		for (idx = 0; idx < cmd->msg_args_len; idx++)
+		for (size_t idx = 0; idx < cmd->msg_args_len; idx++)
 			NLA_PUT(msg, cmd->msg_args[idx].type,
 				     cmd->msg_args[idx].len,
 				     cmd->msg_args[idx].data);
+		free_msg_args(cmd);
 	}
 
 	ret = nl_send_auto_complete(cmd->sk, msg);
@@ -98,6 +109,22 @@ out:
 	return ret;
 }
 
+/**
+ * handle_interface_cmd: handle @cmd for the configured default interface.
+ */
+int handle_interface_cmd(struct cmd *cmd)
+{
+	uint32_t ifindex = if_nametoindex(conf_ifname());
+
+	if (ifindex == 0 && errno)
+		err_sys("failed to look up interface index of '%s'", conf_ifname());
+
+	/* netdev identifier: interface index */
+	add_msg_arg(cmd, NL80211_ATTR_IFINDEX, sizeof(ifindex), &ifindex);
+
+	return handle_cmd(cmd);
+}
+
 /*
  * STATION COMMANDS
  */
@@ -127,7 +154,9 @@ void parse_bitrate(struct nlattr *bitrate_attr, char *buf, int buflen)
 		rate = nla_get_u16(rinfo[NL80211_RATE_INFO_BITRATE]);
 	if (rate > 0)
 		pos += snprintf(pos, buflen - (pos - buf),
-				"%d.%d Mbit/s", rate / 10, rate % 10);
+				"%d.%d MBit/s", rate / 10, rate % 10);
+	else
+		pos += snprintf(pos, buflen - (pos - buf), "(unknown)");
 
 	if (rinfo[NL80211_RATE_INFO_MCS])
 		pos += snprintf(pos, buflen - (pos - buf),
@@ -148,6 +177,21 @@ void parse_bitrate(struct nlattr *bitrate_attr, char *buf, int buflen)
 	if (rinfo[NL80211_RATE_INFO_VHT_NSS])
 		pos += snprintf(pos, buflen - (pos - buf),
 				" VHT-NSS %d", nla_get_u8(rinfo[NL80211_RATE_INFO_VHT_NSS]));
+	if (rinfo[NL80211_RATE_INFO_HE_MCS])
+		pos += snprintf(pos, buflen - (pos - buf),
+				" HE-MCS %d", nla_get_u8(rinfo[NL80211_RATE_INFO_HE_MCS]));
+	if (rinfo[NL80211_RATE_INFO_HE_NSS])
+		pos += snprintf(pos, buflen - (pos - buf),
+				" HE-NSS %d", nla_get_u8(rinfo[NL80211_RATE_INFO_HE_NSS]));
+	if (rinfo[NL80211_RATE_INFO_HE_GI])
+		pos += snprintf(pos, buflen - (pos - buf),
+				" HE-GI %d", nla_get_u8(rinfo[NL80211_RATE_INFO_HE_GI]));
+	if (rinfo[NL80211_RATE_INFO_HE_DCM])
+		pos += snprintf(pos, buflen - (pos - buf),
+				" HE-DCM %d", nla_get_u8(rinfo[NL80211_RATE_INFO_HE_DCM]));
+	if (rinfo[NL80211_RATE_INFO_HE_RU_ALLOC])
+		pos += snprintf(pos, buflen - (pos - buf),
+				" HE-RU-ALLOC %d", nla_get_u8(rinfo[NL80211_RATE_INFO_HE_RU_ALLOC]));
 }
 
 /*
@@ -156,7 +200,7 @@ void parse_bitrate(struct nlattr *bitrate_attr, char *buf, int buflen)
 void print_ssid_escaped(char *buf, const size_t buflen,
 			const uint8_t *data, const size_t datalen)
 {
-	int i, l;
+	size_t i, l;
 
 	memset(buf, '\0', buflen);
 	/* Treat zeroed-out SSIDs separately */
@@ -414,10 +458,10 @@ static int link_handler(struct nl_msg *msg, void *arg)
 	static struct nla_policy bss_policy[NL80211_BSS_MAX + 1] = {
 		[NL80211_BSS_TSF] = { .type = NLA_U64 },
 		[NL80211_BSS_FREQUENCY] = { .type = NLA_U32 },
-		[NL80211_BSS_BSSID] = { },
+		[NL80211_BSS_BSSID] = { 0 },
 		[NL80211_BSS_BEACON_INTERVAL] = { .type = NLA_U16 },
 		[NL80211_BSS_CAPABILITY] = { .type = NLA_U16 },
-		[NL80211_BSS_INFORMATION_ELEMENTS] = { },
+		[NL80211_BSS_INFORMATION_ELEMENTS] = { 0 },
 		[NL80211_BSS_SIGNAL_MBM] = { .type = NLA_U32 },
 		[NL80211_BSS_SIGNAL_UNSPEC] = { .type = NLA_U8 },
 		[NL80211_BSS_STATUS] = { .type = NLA_U32 },
@@ -618,15 +662,9 @@ void iw_nl80211_get_linkstat(struct iw_nl80211_linkstat *ls)
 		.handler = link_sta_handler
 	};
 
-	struct msg_attribute station_addr = {
-		.type = NL80211_ATTR_MAC,
-		.len  = sizeof(ls->bssid),
-		.data = &ls->bssid
-	};
-
 	cmd_linkstat.handler_arg = ls;
 	memset(ls, 0, sizeof(*ls));
-	handle_cmd(&cmd_linkstat);
+	handle_interface_cmd(&cmd_linkstat);
 
 	/* If not associated to another station, the bssid is zeroed out */
 	if (ether_addr_is_zero(&ls->bssid))
@@ -635,10 +673,9 @@ void iw_nl80211_get_linkstat(struct iw_nl80211_linkstat *ls)
 	 * Details of the associated station
 	 */
 	cmd_getstation.handler_arg  = ls;
-	cmd_getstation.msg_args     = &station_addr;
-	cmd_getstation.msg_args_len = 1;
+	add_msg_arg(&cmd_getstation, NL80211_ATTR_MAC, sizeof(ls->bssid), &ls->bssid);
 
-	handle_cmd(&cmd_getstation);
+	handle_interface_cmd(&cmd_getstation);
 
 	/* Channel survey data */
 	iw_nl80211_get_survey(&ls->survey);
@@ -654,7 +691,7 @@ void iw_nl80211_getreg(struct iw_nl80211_reg *ir)
 
 	cmd_reg.handler_arg = ir;
 	memset(ir, 0, sizeof(*ir));
-	handle_cmd(&cmd_reg);
+	handle_interface_cmd(&cmd_reg);
 }
 
 /** Check kernel for split-wiphy support. Single-thread use only. */
@@ -669,7 +706,7 @@ static bool iw_nl80211_have_split_wiphy_dump()
 	};
 
 	if (!checked)
-		handle_cmd(&cmd_features);
+		handle_interface_cmd(&cmd_features);
 	checked = true;
 	return nl80211_has_split_wiphy;
 }
@@ -684,7 +721,7 @@ void iw_nl80211_getifstat(struct iw_nl80211_ifstat *ifs)
 
 	cmd_ifstat.handler_arg = ifs;
 	memset(ifs, 0, sizeof(*ifs));
-	handle_cmd(&cmd_ifstat);
+	handle_interface_cmd(&cmd_ifstat);
 }
 
 void iw_nl80211_get_phy(struct iw_nl80211_ifstat *ifs)
@@ -702,7 +739,7 @@ void iw_nl80211_get_phy(struct iw_nl80211_ifstat *ifs)
 	}
 	cmd_phy_info.handler_arg = ifs;
 	memset(&ifs->phy, 0, sizeof(ifs->phy));
-	handle_cmd(&cmd_phy_info);
+	handle_interface_cmd(&cmd_phy_info);
 }
 
 void iw_nl80211_get_power_save(struct iw_nl80211_ifstat *ifs) {
@@ -714,7 +751,7 @@ void iw_nl80211_get_power_save(struct iw_nl80211_ifstat *ifs) {
 
 	cmd_power_save_info.handler_arg = ifs;
 	ifs->power_save = false;
-	handle_cmd(&cmd_power_save_info);
+	handle_interface_cmd(&cmd_power_save_info);
 }
 
 void iw_nl80211_get_survey(struct iw_nl80211_survey *sd)
@@ -727,7 +764,7 @@ void iw_nl80211_get_survey(struct iw_nl80211_survey *sd)
 
 	cmd_survey.handler_arg = sd;
 	memset(sd, 0, sizeof(*sd));
-	handle_cmd(&cmd_survey);
+	handle_interface_cmd(&cmd_survey);
 }
 
 /*
