@@ -1,12 +1,12 @@
 /*
  * Definitions and functions for nl80211 based routines.
  */
-#include <netlink/netlink.h>
-#include <netlink/genl/genl.h>
-#include <netlink/genl/ctrl.h>
-
-#include <netinet/ether.h>
 #include <stdbool.h>
+#include <netinet/ether.h>
+
+#include <netlink/netlink.h>
+#include <netlink/cli/addr.h>
+#include <netlink/cli/link.h>
 
 /*
  * Use local copy of nl80211.h rather than the one shipped with the distro in
@@ -27,13 +27,13 @@
  * @data:	pointer to data area of length @len
  */
 struct msg_attribute {
-	int		type,
-			len;
+	int		type;
+	size_t		len;
 	const void	*data;
 };
 
 /**
- * struct cmd - stolen and modified from iw:iw.h
+ * struct cmd - represent a single nl80211 command
  * @cmd:	  nl80211 command to send via GeNetlink
  * @sk:		  netlink socket to be used for this command
  * @flags:	  flags to set in the GeNetlink message
@@ -46,15 +46,18 @@ struct msg_attribute {
 struct cmd {
 	enum nl80211_commands	cmd;
 	struct nl_sock		*sk;
+
 	int			flags,
 				hdr_flags;
+
 	int (*handler)(struct nl_msg *msg, void *arg);
 	void			*handler_arg;
 
 	struct msg_attribute	*msg_args;
-	uint8_t			msg_args_len;
+	size_t			msg_args_len;
 };
 extern int handle_cmd(struct cmd *cmd);
+extern int handle_interface_cmd(struct cmd *cmd);
 
 
 /**
@@ -98,8 +101,8 @@ struct iw_nl80211_ifstat {
 
 	char		ssid[64];
 
-	uint32_t	freq;
-	int		chan_width,
+	uint32_t	freq,
+			chan_width,
 			chan_type,
 			freq_ctr1,
 			freq_ctr2;
@@ -273,29 +276,30 @@ enum plink_state {
 };
 
 /* Predefined handlers, stolen from iw:iw.c */
-static inline int error_handler(struct sockaddr_nl *nla, struct nlmsgerr *err,
-			 void *arg)
+static inline int error_handler(struct sockaddr_nl __attribute__((unused))*nla,
+				struct nlmsgerr *err, void *arg)
 {
 	int *ret = arg;
 	*ret = err->error;
 	return NL_STOP;
 }
 
-static inline int finish_handler(struct nl_msg *msg, void *arg)
+static inline int finish_handler(struct nl_msg __attribute__((unused))*msg, void *arg)
 {
 	int *ret = arg;
 	*ret = 0;
 	return NL_SKIP;
 }
 
-static inline int ack_handler(struct nl_msg *msg, void *arg)
+static inline int ack_handler(struct nl_msg __attribute__((unused))*msg, void *arg)
 {
 	int *ret = arg;
 	*ret = 0;
 	return NL_STOP;
 }
 
-static inline int no_seq_check(struct nl_msg *msg, void *arg)
+static inline int no_seq_check(struct nl_msg __attribute__((unused))*msg,
+			       void __attribute__((unused))*arg)
 {
 	return NL_OK;
 }
