@@ -38,6 +38,9 @@
 /* GLOBALS */
 static WINDOW *w_lhist, *w_key;
 
+/* Keep track of interface changes. */
+static int last_if_idx = -1;
+
 /*
  *	Keeping track of global minima/maxima
  */
@@ -45,7 +48,7 @@ static struct iw_extrema {
 	bool	initialised;
 	float	min;
 	float	max;
-} e_signal, e_noise, e_snr;
+} e_signal;
 
 static void track_extrema(const float new_sample, struct iw_extrema *ie)
 {
@@ -89,7 +92,7 @@ static char *fmt_extrema(const struct iw_extrema *ie, const char *unit)
 #define IW_STACKSIZE		1024
 static struct iw_levelstat	iw_stats_cache[IW_STACKSIZE];
 static uint32_t			count;
-const static uint32_t		COUNTMAX = ~0;
+static const uint32_t		COUNTMAX = ~0;
 
 static void iw_cache_insert(const struct iw_levelstat new)
 {
@@ -105,7 +108,7 @@ static void iw_cache_insert(const struct iw_levelstat new)
 
 static struct iw_levelstat iw_cache_get(const uint32_t index)
 {
-	struct iw_levelstat zero = IW_LSTAT_INIT;
+	struct iw_levelstat zero = {.signal = 0, .valid = false};
 
 	if (index > IW_STACKSIZE || index > count)
 		return zero;
@@ -114,7 +117,7 @@ static struct iw_levelstat iw_cache_get(const uint32_t index)
 
 void iw_cache_update(struct iw_nl80211_linkstat *ls)
 {
-	static struct iw_levelstat prev, avg = IW_LSTAT_INIT;
+	static struct iw_levelstat avg = {.signal = 0, .valid = false};
 	static int slot;
 	int sig_level = ls->signal;
 
@@ -135,37 +138,23 @@ void iw_cache_update(struct iw_nl80211_linkstat *ls)
 	if (sig_level == 0)
 		sig_level = ls->bss_signal;
 
+	/* If the signal level is positive, assume it is an absolute value (#100). */
+	if (sig_level > 0)
+		sig_level *= -1;
+
 	if (sig_level == 0) {
-		avg.flags  |= IW_QUAL_LEVEL_INVALID;
+		avg.valid = false;
 	} else {
-		avg.flags  &= ~IW_QUAL_LEVEL_INVALID;
+		avg.valid = true;
 		avg.signal += (float)sig_level / conf.slotsize;
 		track_extrema(sig_level, &e_signal);
 	}
 
-	if (iw_nl80211_have_survey_data(ls)) {
-		avg.flags  &= ~IW_QUAL_NOISE_INVALID;
-		avg.noise += (float)ls->survey.noise / conf.slotsize;
-		track_extrema(ls->survey.noise, &e_noise);
-		if (! (avg.flags & IW_QUAL_LEVEL_INVALID))
-			track_extrema(sig_level - ls->survey.noise, &e_snr);
-	}
-
 	if (++slot >= conf.slotsize) {
 		iw_cache_insert(avg);
 
-		if (conf.lthreshold_action &&
-		    prev.signal < conf.lthreshold &&
-		    avg.signal >= conf.lthreshold)
-			threshold_action(conf.lthreshold);
-		else if (conf.hthreshold_action &&
-			 prev.signal > conf.hthreshold &&
-			 avg.signal <= conf.hthreshold)
-			threshold_action(conf.hthreshold);
-
-		prev = avg;
-		avg.signal = avg.noise = slot = 0;
-		avg.flags  = IW_QUAL_LEVEL_INVALID | IW_QUAL_NOISE_INVALID;
+		avg.signal = slot = 0;
+		avg.valid  = false;
 	}
 }
 
@@ -197,38 +186,22 @@ static int hist_x(int xval)
 /* plot single values, without clamping to min/max */
 static void hist_plot(double yval, int xval, enum colour_pair plot_colour)
 {
-	double level, fraction;
-	chtype ch;
-
-	fraction = modf(yval, &level);
+	int level = round(yval);
 
 	if (in_range(level, 1, HIST_MAXYLEN)) {
-		/*
-		 * The 5 different scanline chars provide a pretty good accuracy.
-		 * ncurses will fall back to standard ASCII chars anyway if they
-		 * are not available.
-		 */
-		if (fraction < 0.2)
-			ch = ACS_S9;
-		else if (fraction < 0.4)
-			ch = ACS_S7;
-		else if (fraction < 0.6)
-			ch = ACS_HLINE;
-		else if (fraction < 0.8)
-			ch = ACS_S3;
-		else
-			ch = ACS_S1;
-
 		wattrset(w_lhist, COLOR_PAIR(plot_colour) | A_BOLD);
-		mvwaddch(w_lhist, hist_y(level), hist_x(xval), ch);
+#ifdef HAVE_LIBNCURSESW
+		mvwadd_wch(w_lhist, hist_y(level), hist_x(xval), WACS_HLINE);
+#else
+		mvwaddch(w_lhist, hist_y(level), hist_x(xval), ACS_HLINE);
+#endif
 	}
 }
 
 static void display_lhist(void)
 {
 	struct iw_levelstat iwl;
-	double snr_level, noise_level, sig_level;
-	enum colour_pair plot_colour;
+	double sig_level;
 	int x, y;
 
 	for (x = 1; x <= MAXXLEN; x++) {
@@ -240,35 +213,11 @@ static void display_lhist(void)
 		for (y = 1; y <= HIST_MAXYLEN; y++)
 			mvwaddch(w_lhist, hist_y(y), hist_x(x), (y % 5) ? ' ' : '-');
 
-		/*
-		 * SNR comes first, as it determines the background. If either
-		 * noise or signal is invalid, set level below minimum value to
-		 * indicate that no background is present.
-		 */
-		if (iwl.flags & (IW_QUAL_NOISE_INVALID | IW_QUAL_LEVEL_INVALID)) {
-			snr_level = 0;
-		} else {
-			snr_level = hist_level(iwl.signal - iwl.noise,
-					       conf.sig_min - conf.noise_max,
-					       conf.sig_max - conf.noise_min);
-
-			wattrset(w_lhist, COLOR_PAIR(CP_BLUE_ON_BLUE));
-			for (y = 1; y <= clamp(snr_level, 1, HIST_MAXYLEN); y++)
-				mvwaddch(w_lhist, hist_y(y), hist_x(x), ' ');
-		}
-
-		if (! (iwl.flags & IW_QUAL_NOISE_INVALID)) {
-			noise_level = hist_level(iwl.noise, conf.noise_min, conf.noise_max);
-			plot_colour = noise_level > snr_level ? CP_RED : CP_RED_ON_BLUE;
-			hist_plot(noise_level, x, plot_colour);
-
-		} else if (x == LEVEL_TAG_POS && ! (iwl.flags & IW_QUAL_LEVEL_INVALID)) {
+		if (x == LEVEL_TAG_POS && iwl.valid) {
 			char	tmp[LEVEL_TAG_POS + 1];
 			int	len;
 			/*
 			 * Tag the horizontal grid lines with dBm levels.
-			 * This is only supported for signal levels, when the screen is not
-			 * shared by several graphs (each having a different scale).
 			 */
 			wattrset(w_lhist, COLOR_PAIR(CP_GREEN));
 			for (y = 1; y <= HIST_MAXYLEN; y++) {
@@ -281,10 +230,9 @@ static void display_lhist(void)
 			}
 		}
 
-		if (! (iwl.flags & IW_QUAL_LEVEL_INVALID)) {
-			sig_level   = hist_level(iwl.signal, conf.sig_min, conf.sig_max);
-			plot_colour = sig_level > snr_level ? CP_GREEN : CP_GREEN_ON_BLUE;
-			hist_plot(sig_level, x, plot_colour);
+		if (iwl.valid) {
+			sig_level = hist_level(iwl.signal, conf.sig_min, conf.sig_max);
+			hist_plot(sig_level, x, CP_GREEN);
 		}
 	}
 
@@ -293,7 +241,7 @@ static void display_lhist(void)
 
 static void display_key(WINDOW *w_key)
 {
-	char buf[128];
+	char buf[280];
 	/* Clear the (one-line) screen) */
 	wmove(w_key, 1, 1);
 	wclrtoborder(w_key);
@@ -304,27 +252,8 @@ static void display_key(WINDOW *w_key)
 	waddch(w_key, ACS_HLINE);
 	wattrset(w_key, COLOR_PAIR(CP_STANDARD));
 
-	if (e_noise.initialised && e_snr.initialised) {
-		snprintf(buf, sizeof(buf), "] sig lvl (%s)  [", fmt_extrema(&e_signal, "dBm"));
-		waddstr(w_key, buf);
-
-		wattrset(w_key, COLOR_PAIR(CP_RED));
-		waddch(w_key, ACS_HLINE);
-		wattrset(w_key, COLOR_PAIR(CP_STANDARD));
-
-		snprintf(buf, sizeof(buf), "] ns lvl (%s)  [", fmt_extrema(&e_noise, "dBm"));
-		waddstr(w_key, buf);
-
-		wattrset(w_key, COLOR_PAIR(CP_BLUE_ON_BLUE));
-		waddch(w_key, ' ');
-
-		wattrset(w_key, COLOR_PAIR(CP_STANDARD));
-		snprintf(buf, sizeof(buf), "] S-N ratio (%s)", fmt_extrema(&e_snr, "dB"));
-		waddstr(w_key, buf);
-	} else {
-		snprintf(buf, sizeof(buf), "] signal level (%s)", fmt_extrema(&e_signal, "dBm"));
-		waddstr(w_key, buf);
-	}
+	snprintf(buf, sizeof(buf), "] signal level (%s)", fmt_extrema(&e_signal, "dBm"));
+	waddstr(w_key, buf);
 
 	wrefresh(w_key);
 }
@@ -334,6 +263,11 @@ void scr_lhist_init(void)
 	w_lhist = newwin_title(0, HIST_WIN_HEIGHT, "Level histogram", true);
 	w_key   = newwin_title(HIST_MAXYLEN + 1, KEY_WIN_HEIGHT, "Key", false);
 
+	if (last_if_idx != conf.if_idx) {
+		count = 0;
+		e_signal.initialised = false;
+		last_if_idx = conf.if_idx;
+	}
 	sampling_init(true);
 
 	display_key(w_key);
