@@ -23,7 +23,6 @@ static struct scan_result sr = {
 	.head          = NULL,
 	.channel_stats = NULL,
 	.msg[0]        = '\0',
-	.mutex         = PTHREAD_MUTEX_INITIALIZER,
 };
 static pthread_t scan_thread;
 static WINDOW *w_aplst;
@@ -96,15 +95,15 @@ static void fmt_scan_entry(struct scan_entry *cur, char buf[], size_t buflen)
 
 static void display_aplist(WINDOW *w_aplst)
 {
-	char s[IW_ESSID_MAX_SIZE << 3];
+	char s[256];
 	const char *sort_type[] = {
-		[SO_CHAN]	= "Chan",
-		[SO_SIGNAL]	= "Sig",
-		[SO_MAC]        = "Mac",
-		[SO_ESSID]	= "Essid",
-		[SO_OPEN]	= "Open",
-		[SO_CHAN_SIG]	= "Ch/Sg",
-		[SO_OPEN_SIG]	= "Op/Sg"
+		[SO_CHAN]     = "Chan",
+		[SO_SIGNAL]   = "Sig",
+		[SO_MAC]      = "Mac",
+		[SO_ESSID]    = "Essid",
+		[SO_OPEN]     = "Open",
+		[SO_CHAN_SIG] = "Ch/Sg",
+		[SO_OPEN_SIG] = "Op/Sg"
 	};
 	int i, col, line = 1;
 	struct scan_entry *cur;
@@ -202,7 +201,7 @@ static void display_aplist(WINDOW *w_aplst)
 			sprintf(s, "top-%d:", (int)sr.num.ch_stats);
 		wadd_attr_str(w_aplst, A_REVERSE, s);
 
-		for (i = 0; i < sr.num.ch_stats; i++) {
+		for (size_t i = 0; i < sr.num.ch_stats; i++) {
 			waddstr(w_aplst, i ? ", " : " ");
 			sprintf(s, "ch#%d", sr.channel_stats[i].val);
 			wadd_attr_str(w_aplst, A_BOLD, s);
@@ -217,12 +216,22 @@ done:
 
 void scr_aplst_init(void)
 {
+	static bool initialized = false;
+
 	w_aplst = newwin_title(0, WAV_HEIGHT, "Scan window", false);
 
 	/* Gathering scan data can take seconds. Inform user. */
 	mvwaddstr(w_aplst, 2, 1, "Waiting for scan data ...");
 	wrefresh(w_aplst);
 
+	if (!initialized) {
+		pthread_mutexattr_t attr;
+
+		pthread_mutexattr_init(&attr);
+		pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
+		pthread_mutex_init(&sr.mutex, &attr);
+		initialized = true;
+	}
 	pthread_create(&scan_thread, NULL, do_scan, &sr);
 }
 
@@ -287,6 +296,9 @@ int scr_aplst_loop(WINDOW *w_menu)
 
 void scr_aplst_fini(void)
 {
+	/* Unlock mutex in case it was taken when scr_aplst_loop got interrupted by a SIGWINCH.
+	 * We are ignoring the error (EPERM) here if the main thread did not acquire the mutex. */
+	pthread_mutex_unlock(&sr.mutex);
 	pthread_cancel(scan_thread);
 	pthread_join(scan_thread, NULL);
 	delwin(w_aplst);
