@@ -3,6 +3,7 @@
  */
 #include "wavemon.h"
 #include "nl80211.h"
+#include <stdarg.h>
 #include <fcntl.h>
 #include <arpa/inet.h>
 #include <netinet/in.h>
@@ -15,18 +16,22 @@
 /* Maximum length of a MAC address: 2 * 6 hex digits, 6 - 1 colons, plus '\0' */
 #define MAC_ADDR_MAX	18
 
-/** Read up to @buflen contents of file @path into @buf. Returns -1 on error. */
+/** Read up to @buflen-1 contents of file @path into @buf. Returns -1 on error. */
 ssize_t read_file(const char *path, char *buf, size_t buflen) {
-	int fd = open(path, O_RDONLY), n;
+	int fd = open(path, O_RDONLY), i, n;
 
 	if (fd < 0)
 		return -1;
 
-	memset(buf, 0, buflen);
+	buf[buflen-1] = '\0';
 
-	n = read(fd, buf, buflen);
+	n = read(fd, buf, buflen-1);
 	if (close(fd) < 0)
 		return -1;
+
+	// Strip trailing whitespace.
+	for (i = n-1; i >= 0 && isspace(buf[i]); i--)
+		buf[i] = '\0';
 	return n;
 }
 
@@ -80,6 +85,33 @@ char *mac_addr(const struct sockaddr *sa)
 	return ether_lookup((const struct ether_addr *)sa->sa_data);
 }
 
+
+/* Print into allocated string (reimplementation of asprintf(3), since not standard). */
+char *a_sprintf(const char *fmt, ...)
+{
+	char *p = NULL;
+	int size;
+	va_list ap;
+
+	va_start(ap, fmt);
+	size = vsnprintf(NULL, 0, fmt, ap);
+	if (size < 0)
+		err_quit("unable to determine '%s' string size", fmt);
+	va_end(ap);
+
+	p = malloc(++size);	/* Add one for '\0'. */
+	if (p == NULL)
+		err_sys("unable to allocate string space");
+
+	va_start(ap, fmt);
+	size = vsnprintf(p, size, fmt, ap);
+	if (size < 0)
+		err_quit("unable to format '%s' properly", fmt);
+	va_end(ap);
+
+	return p;
+}
+
 /* count bits set in @mask the Brian Kernighan way */
 uint8_t bit_count(uint32_t mask)
 {
@@ -92,9 +124,15 @@ uint8_t bit_count(uint32_t mask)
 }
 
 /* netmask = contiguous 1's followed by contiguous 0's */
-uint8_t prefix_len(const struct in_addr *netmask)
+uint8_t prefix_len(const struct sockaddr *netmask)
 {
-	return bit_count(netmask->s_addr);
+	uint8_t bs = 0;
+
+	if (netmask->sa_family == AF_INET)
+		return bit_count(((const struct sockaddr_in *)netmask)->sin_addr.s_addr);
+	for (int i = 0; i < 16; i++)
+		bs += bit_count(((const struct sockaddr_in6 *)netmask)->sin6_addr.s6_addr[i]);
+	return bs;
 }
 
 /* Pretty-print @sec into a string of up to 6 characters. */
@@ -137,6 +175,14 @@ const char *pretty_time_ms(const unsigned msec)
 	return pretty_time(msec/1000);
 }
 
+/* Convert (preferred, valid) address lifetime to string. */
+const char *lft2str(const uint32_t lifetime)
+{
+	if (lifetime == (uint32_t)-1)
+		return "forever";
+	return pretty_time(lifetime);
+}
+
 /* Convert log dBm values to linear mW */
 double dbm2mw(const double in)
 {
@@ -199,6 +245,10 @@ const char *channel_width_name(enum nl80211_chan_width width)
 		return "80+80 MHz";
 	case NL80211_CHAN_WIDTH_160:
 		return "160 MHz";
+	case NL80211_CHAN_WIDTH_5:
+		return "5 MHz";
+	case NL80211_CHAN_WIDTH_10:
+		return "10 MHz";
 	default:
 		return "unknown";
 	}
