@@ -32,9 +32,6 @@
 #include <math.h>
 #include <stdint.h>
 #include <stdbool.h>
-#ifdef HAVE_LIBNCURSESW
-#define _XOPEN_SOURCE_EXTENDED
-#endif
 #if defined(HAVE_LIBNCURSESW) && defined(HAVE_NCURSESW_CURSES_H)
 #include <ncursesw/curses.h>
 #else
@@ -43,8 +40,6 @@
 
 #include "llist.h"
 
-#define CFNAME	".wavemonrc"
-
 /**
  * Minimum screen dimensions.
  * The number of lines depends on the size requirements of scr_info(). The
@@ -53,18 +48,16 @@
  * screen size, in particular allowing the use on the console.
  */
 enum info_screen_geometry {
-	WH_IFACE    = 2,	/* 'Interface' area at the top */
-	WH_LEVEL    = 9,	/* Level meters */
-	WH_STATS    = 3,	/* WiFi statistics area */
-	WH_INFO     = 8,	/* WiFi information area */
-	WH_NET_MIN  = 3,	/* Network interface information area */
-	WH_NET_MAX  = 5,	/* Network interface information area */
-	WH_MENU	    = 1		/* Menu bar at the bottom */
+	WH_IFACE = 2,	/* 'Interface' area at the top */
+	WH_LEVEL = 9,	/* Level meters */
+	WH_STATS = 3,	/* WiFi statistics area */
+	WH_INFO  = 8,	/* WiFi information area */
+	WH_NET   = 7,	/* Network interface information area */
+	WH_MENU  = 1	/* Menu bar at the bottom */
 };
 #define WH_INFO_SCR_BASE	(WH_IFACE + WH_LEVEL + WH_STATS + WH_MENU)
-#define WH_INFO_SCR_MIN		(WH_INFO_SCR_BASE + WH_INFO + WH_NET_MIN)
 
-#define MIN_SCREEN_LINES	WH_INFO_SCR_MIN
+#define MIN_SCREEN_LINES	(WH_INFO_SCR_BASE + WH_INFO + WH_NET)
 #define MIN_SCREEN_COLS		80
 /*
  * Screen layout constants.
@@ -90,27 +83,8 @@ enum info_screen_geometry {
 /* Number of seconds to display a warning message outside ncurses mode */
 #define WARN_DISPLAY_DELAY	1
 
-/* Minimum SSID length */
-#define MAX_ESSID_LEN		16
-
-/*
- * Symbolic names of actions to take when crossing thresholds.
- * These actions invoke the corresponding ncurses functions.
- */
-enum threshold_action {
-	TA_DISABLED,
-	TA_BEEP,
-	TA_FLASH,
-	TA_BEEP_FLASH
-};
-
-static inline void threshold_action(enum threshold_action action)
-{
-	if (action & TA_FLASH)
-		flash();
-	if (action & TA_BEEP)
-		beep();
-}
+/* Maximum length of a formatted SSID. */
+#define MAX_ESSID_LEN		32
 
 /*
  * Symbolic names for scan sort order comparison.
@@ -174,7 +148,7 @@ extern void getconf(int argc, char *argv[]);
 /* Configuration items to manipulate the current configuration */
 struct conf_item {
 	char	*name,		/* name for preferences screen */
-		*cfname;	/* name for ~/.wavemonrc */
+		*cfname;	/* name for wavemonrc(5) */
 
 	enum {			/* type of parameter */
 		t_int,		/* @v.i is interpreted as raw value */
@@ -190,7 +164,7 @@ struct conf_item {
 
 	char	**list;		/* t_list: NULL-terminated array of strings */
 	int	*dep;		/* dependency */
-	bool	hidden;		/* only allow to set via wavemonrc */
+	bool	hidden;		/* only allow to set via wavemonrc(5) */
 
 	int	min,		/* value boundaries */
 		max,
@@ -255,14 +229,15 @@ static inline void waddstr_b(WINDOW * win, const char *s)
 }
 
 extern void waddstr_center(WINDOW * win, int y, const char *s);
-extern const char *curtail(const char *str, const char *sep, int len);
+extern const char *curtail(const char *str, const char *sep, size_t len);
 
 extern void waddbar(WINDOW *win, int y, float v, float min, float max,
 		    int8_t *cscale, bool rev);
 extern void waddthreshold(WINDOW *win, int y, float v, float tv,
 			  float minv, float maxv, int8_t *cscale, chtype tch);
 enum colour_pair {
-	CP_STANDARD = 1,
+	/* CP_STANDARD must be 0 for transparency to work */
+	CP_STANDARD,
 	CP_RED,
 	CP_YELLOW,
 	CP_GREEN,
@@ -291,9 +266,8 @@ static inline int cp_from_scale(float value, int8_t const *cscale, bool reverse)
 /*
  *	Wireless interfaces
  */
-extern const char *conf_ifname(void);
 extern void conf_get_interface_list(void);
-extern void iw_get_interface_list(char** if_list, size_t max_entries);
+extern const char *conf_ifname(void);
 
 /*
  *	Error handling
@@ -317,9 +291,10 @@ static inline void (*xsignal(int signo, void (*handler)(int)))(int)
 	return old_sa.sa_handler;
 }
 
+/** Return the length of the NULL-terminated array @argv. */
 static inline size_t argv_count(char **argv)
 {
-	int cnt = 0;
+	size_t cnt = 0;
 
 	assert(argv != NULL);
 	while (*argv++)
@@ -327,12 +302,12 @@ static inline size_t argv_count(char **argv)
 	return cnt;
 }
 
+/** Find any entry in @argv whose prefix is @what. */
 static inline int argv_find(char **argv, const char *what)
 {
-	int cnt = argv_count(argv), len, i;
+	const size_t len = strlen(what);
 
-	assert(what != NULL);
-	for (i = 0, len = strlen(what); i < cnt; i++)
+	for (size_t i = 0; argv[i]; i++)
 		if (strncasecmp(argv[i], what, len) == 0)
 			return i;
 	return -1;
@@ -398,12 +373,12 @@ static inline char *int_counts(uint32_t count)
 {
 	static char result[0x10];
 
-	if (count < 1000)
+	if (count < 1e3)
 		sprintf(result, "%u", count);
-	else if (count < 1000000)
-		sprintf(result, "%uk", count/1000);
+	else if (count < 1e9)
+		sprintf(result, "%.0fk", count/1e3);
 	else
-		sprintf(result, "%.lg", (double)count);
+		sprintf(result, "%.1G", (double)count);
 
 	return result;
 }
